package no.ntnu.item.arctis.mariubje.rps.lobbyguest.guest;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

import no.ntnu.item.arctis.mariubje.rps.data.Challenge;
import no.ntnu.item.arctis.mariubje.rps.data.ChatMessage;
import no.ntnu.item.ramses.runtime.Scheduler;

public class Guest {

	JFrame frame;
	JTextField chatInputText;
	JButton chatAllButton;
	JButton chatOpponentsButton;
	JTextArea chatArea;
	JList userJList;
	JButton challengeButton;
	public no.ntnu.item.arctis.mariubje.rps.data.Challenge challengeRequestPending;
	public java.lang.String userName;
	
	private ButtonListener btnListener = new ButtonListener();
	
	private static final String nl = System.getProperty("line.separator");

	public void showLobby() {

		frame = new JFrame();
		frame.setTitle(userName + " : Rock, Paper, Scissors - Lobby");
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
//		frame.setSize(600, 350);
		frame.setLocation(300, 300);
		frame.addWindowListener(new WindowListenerImpl());
		
		JPanel chatInputPanel = new JPanel();
		chatInputText = getChatInputTextField();
		chatAllButton = getChatAllButton();
		chatOpponentsButton = getChatOpponentsButton();
		chatInputPanel.add(chatInputText);
		chatInputPanel.add(chatAllButton);
		chatInputPanel.add(chatOpponentsButton);
		frame.add(chatInputPanel, BorderLayout.PAGE_END);
		
		chatArea = getChatArea();
		frame.add(new JScrollPane(chatArea), BorderLayout.CENTER);
		
		JPanel userListPanel = new JPanel();
		userListPanel.setLayout(new BoxLayout(userListPanel, BoxLayout.Y_AXIS));
		challengeButton = getChallengeButton();
		userJList = getUserJList();
		userListPanel.add(new JScrollPane(userJList));
		userListPanel.add(challengeButton);
		frame.getContentPane().add(userListPanel, BorderLayout.LINE_END);
		
		frame.pack();
		frame.setVisible(true);
	}

	private JTextField getChatInputTextField() {
		JTextField chatInputText = new JTextField();
//		chatInputText.setSize(100, 60);
		chatInputText.setColumns(35);
		chatInputText.addActionListener(btnListener);
		return chatInputText;
	}

	private JButton getChatAllButton() {
		JButton btn = new JButton();
//		btn.setSize(100, 50);
		btn.setText("All");
		btn.addActionListener(btnListener);
		return btn;
	}

	private JButton getChatOpponentsButton() {
		JButton btn = new JButton();
//		btn.setSize(100, 50);
		btn.setText("Opponents");
		btn.addActionListener(btnListener);
		return btn;
	}

	private JList getUserJList() {
		JList l = new JList();
		l.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		l.setVisibleRowCount(8);
		l.setFixedCellWidth(50);
		return l;
	}

	private JTextArea getChatArea() {
		JTextArea ta = new JTextArea();
		ta.setColumns(45);
		ta.setRows(10);
		ta.setEditable(false);
		return ta;
	}

	private JButton getChallengeButton() {
		JButton btn = new JButton();
		btn.setSize(150, 50);
		btn.setText("Challenge");
		btn.addActionListener(btnListener);
		return btn;
	}

	public void updateUserList(List userList) {
		userJList.setListData(userList.toArray());
	}

	public void challengeReceived() {
		System.out.println("I challengeReceived()..");
		new Thread() {
			@Override
			public void run() {
				String c = challengeRequestPending.challenger;
				int response = JOptionPane.showConfirmDialog(frame, c + " is challenging you, accept?", "You have been challenged!", JOptionPane.YES_NO_OPTION);
				if (response == JOptionPane.YES_OPTION) {
					Scheduler.getRuntime(Guest.this).sendToMe(Guest.this, "challengeResponse", Boolean.TRUE);
				}
				else if (response == JOptionPane.NO_OPTION) {
					Scheduler.getRuntime(Guest.this).sendToMe(Guest.this, "challengeResponse", Boolean.FALSE);
				}
			}
		}.start();
	}

	public void updateChat(ChatMessage chat) {
		if (chatArea != null) {
			if (chat.toAll) {
				chatArea.append(chat.from + ": " + chat.message + nl);
			}
			else {
				chatArea.append("*" + chat.from + ": " + chat.message + nl);
			}
			chatArea.setCaretPosition(chatArea.getText().length() - 1);
		}
	}

	public void closeLobby() {
		frame.dispose();
		frame = null;
		chatInputText = null;
		chatAllButton = null;
		chatOpponentsButton = null;
		chatArea = null;
		userJList = null;
		challengeButton = null;
	}
	
	public Challenge getAndClearChallengeResponse() {
		Challenge tmp = challengeRequestPending;
		challengeRequestPending = null;
		return tmp;
	}
	
	private class WindowListenerImpl implements WindowListener {

		@Override
		public void windowActivated(WindowEvent e) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void windowClosed(WindowEvent e) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void windowClosing(WindowEvent e) {
			Scheduler.getRuntime(Guest.this).sendToMe(Guest.this, "userLeaving");
		}

		@Override
		public void windowDeactivated(WindowEvent e) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void windowDeiconified(WindowEvent e) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void windowIconified(WindowEvent e) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void windowOpened(WindowEvent e) {
			// TODO Auto-generated method stub
			
		}
		
	}
	
	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			if ((e.getSource() == chatAllButton || e.getSource() == chatInputText) && !chatInputText.getText().equals("")) {
				ChatMessage c = new ChatMessage();
				c.from = userName;
				c.toAll = true;
				c.message = chatInputText.getText();
				chatInputText.setText("");
				chatInputText.requestFocus();
				Scheduler.getRuntime(Guest.this).sendToMe(Guest.this, "chatMessage", c);
			}
			else if (e.getSource() == chatOpponentsButton && !chatInputText.getText().equals("")) {
				ChatMessage c = new ChatMessage();
				c.from = userName;
				c.toAll = false;
				c.message = chatInputText.getText();
				chatInputText.setText("");
				chatInputText.requestFocus();
				Scheduler.getRuntime(Guest.this).sendToMe(Guest.this, "chatMessage", c);
			}
			else if (e.getSource() == challengeButton) {
				if (userJList.getSelectedIndex() != -1 && !userJList.getSelectedValue().equals(userName)) {
					Challenge c = new Challenge();
					c.challenger = userName;
					c.challenged = (String)userJList.getSelectedValue();
					userJList.clearSelection();
					Scheduler.getRuntime(Guest.this).sendToMe(Guest.this, "challenge", c);
				}
			}
		}
	}
}