package no.ntnu.item.arctis.mariubje.rps.rpsgame.playertwo;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import no.ntnu.item.arctis.mariubje.rps.data.Challenge;
import no.ntnu.item.arctis.mariubje.rps.data.RpsResult;
import no.ntnu.item.ramses.runtime.Scheduler;

public class PlayerTwo {
	JFrame frame;
	String opponent;
	String userName;
	private JLabel scoreLabel;
	private JButton rockButton, paperButton, scissorsButton;
	private JLabel actionYou, actionOpponent;
	
	private int wins = 0, draws = 0, losses = 0;
	
	private ButtonListener btnListener = new ButtonListener();

	public void initialize(Challenge challenge) {
		opponent = challenge.challenger;
		userName = challenge.challenged;

		frame = new JFrame();
		frame.setTitle("RPS Game: " + userName + " vs. " + opponent);
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener(new WindowListenerImpl());
//		frame.setSize(300, 300);
		frame.setLocation(100, 100);

		scoreLabel = getScoreLabel();
		frame.add(scoreLabel, BorderLayout.PAGE_START);
		
		JPanel buttonsPanel = new JPanel(new GridLayout(1, 3));
		rockButton = getRockButton();
		paperButton = getPaperButton();
		scissorsButton = getScissorsButton();
		buttonsPanel.add(rockButton);
		buttonsPanel.add(paperButton);
		buttonsPanel.add(scissorsButton);
		frame.add(buttonsPanel, BorderLayout.PAGE_END);
		
		JPanel actionYouPanel = getActionYouPanel();
		JPanel actionOpponentPanel = getActionOpponentPanel();
		frame.add(actionYouPanel, BorderLayout.LINE_START);
		frame.add(actionOpponentPanel, BorderLayout.LINE_END);
		
		frame.pack();
		frame.setVisible(true);
	}

	private JLabel getScoreLabel() {
		JLabel l = new JLabel();
		l.setAlignmentY(JLabel.RIGHT_ALIGNMENT);
		l.setText("No games played yet.");
		return l;
	}

	private JButton getRockButton() {
		JButton btn = new JButton();
		btn.setText("Rock");
		btn.setAlignmentY(JButton.CENTER_ALIGNMENT);
		btn.addActionListener(btnListener);
		return btn;
	}

	private JButton getPaperButton() {
		JButton btn = new JButton();
		btn.setText("Paper");
		btn.setAlignmentY(JButton.CENTER_ALIGNMENT);
		btn.addActionListener(btnListener);
		return btn;
	}

	private JButton getScissorsButton() {
		JButton btn = new JButton();
		btn.setText("Scissors");
		btn.setAlignmentY(JButton.CENTER_ALIGNMENT);
		btn.addActionListener(btnListener);
		return btn;
	}

	private JPanel getActionYouPanel() {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.add(new JLabel("You:"));
		actionYou = new JLabel("?");
		p.add(actionYou);
		return p;
	}

	private JPanel getActionOpponentPanel() {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.add(new JLabel("Opponent:"));
		actionOpponent = new JLabel("?");
		p.add(actionOpponent);
		return p;
	}

	public void showResult(RpsResult result) {
		actionYou.setText(result.playerTwoAction);
		actionOpponent.setText(result.playerOneAction);
		if (result.winner == null || result.winner.equals("")) {
			draws++;
		}
		else if (result.winner.equals(userName)) {
			wins++;
		}
		else if (result.winner.equals(opponent)) {
			losses++;
		}
	}

	public void updateScoresAndResetBoard() {
		scoreLabel.setText("Wins: " + wins + " Losses: " + losses + " Draws: " + draws);
		actionYou.setText("?");
		actionOpponent.setText("?");
		rockButton.setEnabled(true);
		paperButton.setEnabled(true);
		scissorsButton.setEnabled(true);
	}

	public void stopGame() {
		frame.dispose();
		frame = null;
		scoreLabel = null;
		rockButton = null;
		paperButton = null;
		scissorsButton = null;
		actionYou = null;
		actionOpponent = null;
	}

	private class WindowListenerImpl implements WindowListener {

		public void windowActivated(WindowEvent e) {
		}

		public void windowClosed(WindowEvent e) {
		}

		public void windowClosing(WindowEvent e) {
			Scheduler.getRuntime(PlayerTwo.this).sendToMe(PlayerTwo.this, "ClosingGame");
		}

		public void windowDeactivated(WindowEvent e) {
		}

		public void windowDeiconified(WindowEvent e) {
		}

		public void windowIconified(WindowEvent e) {
		}

		public void windowOpened(WindowEvent e) {
		}
	}

	private class ButtonListener implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			String s = null;
			if (e.getSource() == rockButton) {
				s = "rock";
			}
			else if (e.getSource() == paperButton) {
				s = "paper";
			}
			else if (e.getSource() == scissorsButton) {
				s = "scissors";
			}
			rockButton.setEnabled(false);
			paperButton.setEnabled(false);
			scissorsButton.setEnabled(false);
			Scheduler.getRuntime(PlayerTwo.this).sendToMe(PlayerTwo.this, "GameAction", s);
		}
		
	}
}