package no.ntnu.item.ramses.runtime.serialization;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import no.ntnu.item.ramses.runtime.routing.URL;

import org.ksoap2.SoapEnvelope;
import org.ksoap2.serialization.Marshal;
import org.ksoap2.serialization.SoapObject;
import org.ksoap2.serialization.SoapSerializationEnvelope;
import org.kxml2.kdom.*;
import org.kxml2.io.*;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

public class SoapHandler extends Serializer {

    public static final String NAMESPACE = "http://www.item.ntnu.no/RAMSES_SOAP";

    // Keys to the values in the hashtable returned from a parsed soap message.
    public static final String SENDER = "Sender";
    public static final String RECEIVER = "Receiver";
    public static final String MESSAGE_ID = "Message ID";
    public static final String DATA = "Data object";

    // private final SerializationHandler xml = new SerializationHandler();

//    private static SoapHandler singleton;

    private final SoapEnvelopeFactory soapEnvelopeFactory = new SoapEnvelopeFactory();

    public SoapHandler() {
        // Hidden constructor, use Singleton getInstance-pattern
    }

//    public static SoapHandler getInstance() {
//        if (singleton == null) singleton = new SoapHandler();
//        return singleton;
//    }

    public String getSoapMessageAsString(Object sender, Object receiver, String msgId, Object data)
            throws IOException, XmlPullParserException {
        debug("Entering..", "getSoapMessageAsString");

        SoapEnvelope soap = soapEnvelopeFactory.getSoapEnvelope();

        // SoapEnvelope soap = new SoapEnvelope(SoapEnvelope.VER12);
        //
        // debug(DEBUG_VERBOSE, "Setting headers..", "getSoapMessageAsString");
        // First create the header of the SOAP message, it consists of the URI's
        // of the sending and receiving state machines.
        Element[] header = new Element[2];
        Element e = new Element();

        header[1] = e.createElement(NAMESPACE, "sender");
        if (sender != null)
            header[1].addChild(Node.TEXT, sender.toString());
        else debug("Sender was null.", "getSoapMessageAsString");

        header[0] = e.createElement(NAMESPACE, "receiver");
        if (receiver != null)
            header[0].addChild(Node.TEXT, receiver.toString());
        else debug("Receiver is null.", "getSoapMessageAsString");

        soap.headerOut = header;

        debug("Header(s) set.", "getSoapMessageAsString");

        // Now create the body.

        debug("Adding body.", "getSoapMessageAsString");

        SoapObject so = new SoapObject(NAMESPACE, "signal");
        so.addProperty("messageId", msgId);
        if (data != null) {
            so.addProperty("data", data);
        }

        soap.bodyOut = so;

        // Element msg = e.createElement(KXmlParser.NO_NAMESPACE, msgId);
        // if (data != null) {
        // debug(DEBUG_VERBOSE, "- Adding data to body: " + data,
        // "getSoapMessageAsString");
        // if (xml.canSerialize(data)) {
        // msg.addChild(Node.ELEMENT, xml.serialize(data));
        // }
        // else throw new
        // IllegalArgumentException("Data object not serializable: " +
        // data.getClass().getName());
        // // TOD: Remove (Vector)-casting, only for debugging.
        // // if (data instanceof Vector) msg.addChild(Node.ELEMENT,
        // xml.serialize((Vector)data));
        // // else if (data instanceof Hashtable) msg.addChild(Node.ELEMENT,
        // xml.serialize((Hashtable)data));
        // // else throw new
        // IllegalArgumentException("In debugging mode, type not supported");
        // debug(DEBUG_VERBOSE, "- Data added to body.",
        // "getSoapMessageAsString");
        // }
        //
        // soap.bodyOut = msg;
        //		
        debug("Body added.", "getSoapMessageAsString");
        //		
        debug("Writing SOAP xml..", "getSoapMessageAsString");

        KXmlSerializer x = new KXmlSerializer();
        ByteArrayOutputStream byteOutStream = new ByteArrayOutputStream();
        OutputStreamWriter streamWriter = new OutputStreamWriter(byteOutStream);
        x.setOutput(streamWriter);
        soap.write(x);
        streamWriter.flush();
        String soapOut = byteOutStream.toString();
        // System.out.println(soapOut);

        // KXmlSerializer x = new KXmlSerializer();
        // ByteArrayOutputStream byteOutStream = new ByteArrayOutputStream();
        // OutputStreamWriter streamWriter = new
        // OutputStreamWriter(byteOutStream);
        // x.setOutput(streamWriter);
        // soap.write(x);
        // streamWriter.flush();
        // debug(DEBUG_VERBOSE, "SOAP message created:",
        // "getSoapMessageAsString");
        // debug(DEBUG_VERBOSE, "Some baos data. Count:" +
        // byteOutStream.size());
        // debug(DEBUG_VERBOSE, byteOutStream.toString(),
        // "getSoapMessageAsString");
        debug("Leaving..", "getSoapMessageAsString");
        // return byteOutStream.toString();
        return soapOut;
    }

    public Hashtable parseSoapMessage(String soapMessageAsString) throws IOException,
            XmlPullParserException {

        KXmlParser x = new KXmlParser();
        SoapEnvelope soap = soapEnvelopeFactory.getSoapEnvelope();

        byte[] byteArray = soapMessageAsString.getBytes();
        ByteArrayInputStream byteInputStream = new ByteArrayInputStream(byteArray);
        InputStreamReader streamReader = new InputStreamReader(byteInputStream);
        try {
            x.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, true);
            x.setInput(streamReader);
        }
        catch (XmlPullParserException e) {
            // Ignore, should really never happen?
        }

        try {
            soap.parse(x);
        }
        catch (XmlPullParserException e) {
            e.printStackTrace();
            throw new IOException("Caught PullParserException while parsing soap message.");
        }
        
        // Reconstruct receiver and sender which may have been split into multiple text-nodes
        StringBuffer receiver = new StringBuffer();
        for (int i = 0; i < soap.headerIn[0].getChildCount(); i++) {
            if (soap.headerIn[0].getType(i) == XmlPullParser.TEXT) {
                receiver.append(soap.headerIn[0].getText(i));
            }
        }
        
        StringBuffer sender = new StringBuffer();
        for (int i = 0; i < soap.headerIn[1].getChildCount(); i++) {
            if (soap.headerIn[1].getType(i) == XmlPullParser.TEXT) {
                sender.append(soap.headerIn[1].getText(i));
            }
        }

        Hashtable h = new Hashtable();
        h.put(RECEIVER, new URL(receiver.toString()));
        h.put(SENDER, new URL(sender.toString()));
        h.put(MESSAGE_ID, ((SoapObject)soap.bodyIn).getProperty(0));

        // Now the data object must be parsed as well, if it is present.
        if (((SoapObject)soap.bodyIn).getPropertyCount() > 1) { // ((Node)soap.bodyIn).getElement(0).getChildCount()
            // > 0) {
            h.put(DATA, ((SoapObject)soap.bodyIn).getProperty(1));
        }

        return h;
    }

    private static class SoapEnvelopeFactory {

        public SoapEnvelopeFactory() {
            
        }

//        private static SoapEnvelopeFactory singleton = new SoapEnvelopeFactory();
//
//        public static SoapEnvelopeFactory getInstance() {
//            return singleton;
//        }

        public SoapEnvelope getSoapEnvelope() {
            SoapSerializationEnvelope env = new SoapSerializationEnvelope(SoapEnvelope.VER12);

            registerMarshals(env);

            return env;
        }

        private void registerMarshals(SoapSerializationEnvelope env) {
            for (Enumeration e = MarshalManager.getMarshals().elements(); e.hasMoreElements();) {
                Marshal m = (Marshal)e.nextElement();
                m.register(env);
            }
        }
    }

    public MessageObjectsWrapper deserialize(String serializedMessage) {
        try {
            Hashtable messageObjects = parseSoapMessage(serializedMessage);
            return new MessageObjectsWrapper(messageObjects.get(SENDER),
                    messageObjects.get(RECEIVER), (String)messageObjects.get(MESSAGE_ID),
                    messageObjects.get(DATA));
        }
        catch (IOException e) {
            System.err.println("IOException while deserializing! Returning null.");
            e.printStackTrace();
        }
        catch (XmlPullParserException e) {
            System.err.println("XmlPullParserException while deserializing! Returning null.");
            e.printStackTrace();
        }

        return null;
    }

    public String serialize(Object sender, Object receiver, String signalId, Object data) {
        try {
            return getSoapMessageAsString(sender, receiver, signalId, data);
        }
        catch (IOException e) {
            System.err.println("IOException while serializing! Returning null.");
            e.printStackTrace();
        }
        catch (XmlPullParserException e) {
            System.err.println("XmlPullParserException while serializing! Returning null.");
            e.printStackTrace();
        }

        return null;
    }
}
