package no.ntnu.item.ramses.runtime.logging;

public class Logger {
	
	private static Logger instance;
	
	public static synchronized Logger getLogger() {
		if(instance==null) instance = new Logger();
		return instance;
	}
	
	public static synchronized void setLogger(Logger logger) {
		if(instance!=null) throw new IllegalStateException("The logger is already initialized.");
		instance = logger;
	}
	
	/**
	 * A state machine discarded a message.
	 */
	public void discardedSignal(String signalName, String sender, String receiver, String stateMachineType) {
		System.err.println(" ! DISCARDED SIGNAL.  id: " + signalName);
		System.err.println("                  sender: " + sender);
		System.err.println("                receiver: " + receiver);
		System.err.println("           state machine: " + stateMachineType);	
	}
	
	/**
	 * A component was asked to deliver a message to a part that is not foreseen for this component.
	 */
	public void invalidPartName(String partName, String sender, String receiver, String componentType) {
		System.err.println(" ! INVALID PART REQUESTED");
		System.err.println("                partname: " + partName);		
		System.err.println("                  sender: " + sender);
		System.err.println("                receiver: " + receiver);
		System.err.println("          component type: " + componentType);	
	}

	public void sessionCreationFailed(String partName, String stateMachineType) {
		System.err.println(" ! SESSION CREATION FAILED");
		System.err.println("                partname: " + partName);		
		System.err.println("   parent component type: " + stateMachineType);		
	}
	
	public void communicationException(String message, Throwable t) {
		System.err.println(" ! COMMUNICATION EXCEPTION");
		System.err.println("                 message: " + message);
		t.printStackTrace();
	}
	
	public void messageDroppedDuringRouting(String sender, String receiver, String signalId) {
	    System.err.println(" ! DROPPED SIGNAL.    id: " + signalId);
        System.err.println("                  sender: " + sender);
        System.err.println("                receiver: " + receiver); 
	}

}
