/*
 * Copyright 2006-2008 Sun Microsystems, Inc. All Rights Reserved. DO NOT ALTER
 * OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER This code is free software;
 * you can redistribute it and/or modify it under the terms of the GNU General
 * Public License version 2 only, as published by the Free Software Foundation.
 * This code is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License version 2 for more
 * details (a copy is included in the LICENSE file that accompanied this code).
 * You should have received a copy of the GNU General Public License version 2
 * along with this work; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA Please contact Sun
 * Microsystems, Inc., 16 Network Circle, Menlo Park, CA 94025 or visit
 * www.sun.com if you need additional information or have any questions.
 */

/*
 * Modified by mariubje. Overridden toString(). Overridden equals(Object o).
 * Changed the init() method, since the original was buggy when having a port.
 */

package no.ntnu.item.ramses.runtime.routing;

import java.util.Hashtable;
import java.util.Vector;

/**
 * Parses a string of the form
 * "protocolName://address:port/rest1/rest2/.../restN" <br/>
 * <br/>
 * Shamelessly inspired by com.sun.spot.peripheral.ota.URL from Squawk, but
 * improved and made independent from Squawk for use on all plaforms by Marius
 * Bjerke.
 * 
 * @author randy
 * @author Marius Bjerke
 */
public class URL {

    private String protocol = "";
    private String address = "";
    private String port = "";
    private Vector pathComponents = new Vector();
    private String query = "";
    private Hashtable queryComponents = new Hashtable();
    private String fragment = "";

    public static boolean debugging = false;

    public URL(String url) {
        init(url);
    }

    public static void test() {
        debugging = true;
        new URL("serial:");
        new URL("serial://");
        new URL("remoteprint://ABCD.ADCF.1234.2345:90");
        new URL("http://ABCD.ADCF.1234.2345:2/foo/bar");
        new URL("snort://localhost/foo");
        new URL("snort://localhost/foo/bar/baz");
        new URL("snort://localhost/foo?tada");
        new URL("snort://localhost/foo#todo");
        new URL("snort://localhost/foo?tada#todo");
        new URL("sumsum://localhost/foo?key=value&foo=bar&ola=kari");
        new URL("sumsum://localhost/foo?&");
        debugging = false;
    }

    public void debug(String line) {
        if (debugging) System.out.println("[URL] ----------------: " + line);
    }

    private void init(String url) {
        int nextColon = -1;
        int nextSlash = -1;
        String urlRest = url;

        debug("_______________________________________________________________");
        debug("  init() " + url);

        nextColon = urlRest.indexOf(':');
        if (nextColon == -1) throw new IllegalArgumentException("Couldn't parse URL.");
        protocol = urlRest.substring(0, nextColon);
        debug("  init() protocol " + protocol);
        urlRest = urlRest.substring(nextColon + ":".length());

        if (urlRest.length() < 2)
            return;
        else urlRest = urlRest.substring("//".length());

        nextColon = urlRest.indexOf(':');
        if (nextColon != -1) {
            // We may have a port no.
            address = urlRest.substring(0, nextColon);
            debug("  init() address " + address);
            urlRest = urlRest.substring(nextColon + ":".length());

            nextSlash = urlRest.indexOf('/');
            if (nextSlash == -1) {
                // No slash, only port left? Port can still be ""
                port = urlRest;
                debug(" init() port " + port);
                return;
            }
            else {
                port = urlRest.substring(0, nextSlash);
                debug(" init() port " + port);
                urlRest = urlRest.substring(nextSlash + "/".length());
                debug(" init() ...with remainder " + urlRest);
            }
        }
        else {
            nextSlash = urlRest.indexOf('/');
            if (nextSlash == -1) {
                // No slash or port, only address
                address = urlRest;
                debug("  init() address " + address);
                return;
            }
            else {
                // No port, but min. 1 slash
                address = urlRest.substring(0, nextSlash);
                debug("  init() address " + address);
                urlRest = urlRest.substring(nextSlash + "/".length());
                debug(" init() ...with remainder " + urlRest);
            }
        }

        // Check for query and fragment..
        int qMark = urlRest.indexOf('?');
        int sharp = urlRest.indexOf('#');
        if (qMark != -1) {
            if (sharp != -1) {
                if (qMark > sharp)
                    throw new IllegalArgumentException(
                            "Query (?) should always come before a fragment (#).");
                query = urlRest.substring(qMark + 1, sharp);
                fragment = urlRest.substring(sharp + 1);
                urlRest = urlRest.substring(0, qMark);
            }
            else {
                query = urlRest.substring(qMark + 1);
                urlRest = urlRest.substring(0, qMark);
            }
            parseQuery();
        }
        else if (sharp != -1) {
            fragment = urlRest.substring(sharp + 1);
            urlRest = urlRest.substring(0, sharp);
        }

        debug(" init() query " + query);
        debug(" init() queryComponents " + queryComponents);
        debug(" init() fragment " + fragment);

        while (true) {
            nextSlash = urlRest.indexOf('/');

            if (nextSlash == -1) {
                // No more slash, last element or only empty string left.
                if (urlRest.equals(""))
                    break;
                else {
                    pathComponents.addElement(urlRest);
                    debug(" init() piece " + urlRest);
                    break;
                }
            }
            else {
                // At least one more slash
                String piece = urlRest.substring(0, nextSlash);
                pathComponents.addElement(piece);
                debug(" init() piece " + piece);
                urlRest = urlRest.substring(nextSlash + "/".length());
            }
        }

        debug(" init() parsed: " + toString());
    }

    private void parseQuery() {
        if (query.equals("")) return;
        String q = query;
        String temp;
        int i;

        i = q.indexOf('&');
        while (i != -1) {
            temp = q.substring(0, i);
            if (temp.length() > 0) {
                parseKeyValuePair(temp);
            }

            q = q.substring(i + 1);
            i = q.indexOf('&');
        }
        parseKeyValuePair(q);
    }

    private void parseKeyValuePair(String s) {
        String key;
        String value;
        int i;
        i = s.indexOf('=');
        if (i == -1) {
            if (s.length() > 0) queryComponents.put(s, "");
            return;
        }
        key = s.substring(0, i);
        value = s.substring(i + 1);

        if (key.equals("")) {
            // ignore.
        }
        else {
            queryComponents.put(key, value);
        }
    }

    private String removeStartingString(String s, String c) {
        while (s.startsWith(c)) {
            if (s.length() > c.length()) s = s.substring(c.length());
        }
        return s;
    }

    public String getProtocol() {
        return protocol;
    }

    public String getAddress() {
        return address;
    }

    public String getPort() {
        return port;
    }

    public Vector getPathComponents() {
        return pathComponents;
    }

    public static void setDebugging(boolean b) {
        debugging = b;
    }

    public boolean equals(Object obj) {
        if (obj == this) return true;
        
        if (obj instanceof URL) {
            return equals((URL)obj);
        }
        return false;
    }

    public boolean equals(URL url) {
        if (url == this) return true;
        
        if (!this.getProtocol().equals(url.getProtocol())) return false;
        if (!this.getAddress().equals(url.getAddress())) return false;
        if (!this.getPort().equals(url.getPort())) return false;
        if (this.getPathComponents().size() != url.getPathComponents().size()) return false;
        for (int i = 0; i < this.getPathComponents().size(); i++) {
            if (!this.getPathComponents().elementAt(i).equals(url.getPathComponents().elementAt(i)))
                return false;
        }
        if (!this.getQuery().equals(url.getQuery())) return false;
        return true;
    }

    public int hashCode() {
        return (this.getClass().getName() + ":" + toString()).hashCode();
    }

    public String toString() {
        StringBuffer b = new StringBuffer();
        b.append(this.getProtocol());
        b.append("://");
        b.append(this.getAddress());
        if (this.getPort() != "") {
            b.append(":");
            b.append(this.getPort());
        }
        for (int i = 0; i < this.getPathComponents().size(); i++) {
            b.append("/");
            b.append(this.getPathComponents().elementAt(i));
        }
        if (query != "") {
            b.append("?");
            b.append(query);
        }
        if (fragment != "") {
            b.append("#");
            b.append(fragment);
        }
        return b.toString();
    }

    public String getQuery() {
        return query;
    }
    
    public Hashtable getQueryMap() {
        return queryComponents;
    }

    public String getFragment() {
        return fragment;
    }
}
